define([
    'dojo/i18n!html/nls/STMResources',
    'dojo/text!html/app/view/manageforms/ManageDataEntryFormsAppView.html',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/when',
    'dojo/date',
    'dojo/string',
    'dojo/date/locale',
    'dojo/topic',
    'dojo/data/ObjectStore',
    'dojo/store/Memory',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetsInTemplateMixin',
    'dojox/widget/DialogSimple',
    'dijit/registry',
    'dijit/Toolbar',
    'dijit/ToolbarSeparator',
    'dijit/layout/BorderContainer',
    'dijit/layout/ContentPane',
    'dijit/form/Button',
    'dojox/grid/DataGrid',
    'STM/view/DialogTemplate',
    'STM/model/manageforms/ManageFormsRequests',
    'STM/util/ApplicationConstants',
    'STM/util/manageforms/ApplicationConstants'
], function(resources, templateString, declare, array, when, date, string, locale, topic, ObjectStore, Memory,
            _TemplatedWidget, _WidgetsInTemplateMixin, DialogSimple, registry, Toolbar, ToolbarSeparator, BorderContainer,
            ContentPane, Button, DataGrid, DialogTemplate, ManageFormsRequests, GeneralAppConst, AppConst) {

    //
    // Private getters for convenience
    //

    var getNewButton = function() {
        return registry.byId('toolbarNew');
    };

    var getEditButton = function() {
        return registry.byId('toolbarEditForm');
    };

    var getCopyButton = function() {
        return registry.byId('toolbarCopy');
    };

    var getDeleteButton = function() {
        return registry.byId('toolbarDelete');
    };

    var getStartWorkflowButton = function() {
        return registry.byId('toolbarStartWorkflow');
    };

    var getStopRepeatButton = function() {
        return registry.byId('toolbarStopRepeat');
    };

    var getAbortWorkflowButton = function() {
        return registry.byId('toolbarAbortWorkflow');
    };

    var getRefreshFormsButton = function() {
        return registry.byId('toolbarRefreshForms');
    };
    var substr = "<a href='" + GeneralAppConst.WEBDOC_HTML5_USER_GUIDE_FORM_URL + "' target=\"_blank\">" + resources['de.form.popup.sub.txt'] + "</a>";

    templateString = string.substitute(templateString, {
        toolbarNewTooltipLabel: resources['de.form.toolbar.new.txt'],
        toolbarEditTooltipLabel: resources['de.form.toolbar.edit.txt'],
        toolbarCopyTooltipLabel: resources['de.form.toolbar.copy.txt'],
        toolbarDeleteTooltipLabel: resources['de.form.toolbar.delete.txt'],
        toolbarStartTooltipLabel: resources['de.form.toolbar.startworkflow.txt'],
        toolbarStopTooltipLabel: resources['de.form.toolbar.stoprepeat.txt'],
        toolbarAbortTooltipLabel: resources['de.form.toolbar.abortworkflow.txt'],
        toolbarRefreshFormsLabel: resources['de.form.toolbar.refreshforms.txt'],
        helplabel: resources['help.txt'],
    	helpFormTeaser: resources['de.form.teaser.txt'],
        helpFormContext: string.substitute(resources['de.form.popup.txt'], [substr] ),
        helpText: resources['de.form.helptip.txt']
    });

    return declare([BorderContainer, _TemplatedWidget, _WidgetsInTemplateMixin], {
    //    'class': 'stmFixedHeightContent',
        templateString: templateString,
        //
        // Overrides and init stuff
        //
        constructor: function(args) {
            args['id'] = 'manageFormsAppView';
        },
        postCreate: function() {
            var me = this;

            me.initUI();
        },
        initUI: function() {
            var me = this;

            when(ManageFormsRequests.getDataEntryList(), function(forms) {
                var store = new ObjectStore({
                    objectStore: new Memory({data: forms})
                });

                var grid = new DataGrid({
                        store: store,
                        structure: [
                            {
                                name: resources['de.form.name.column.txt'],
                                field: 'name',
                                width: '100px'
                            },
                            {
                                name: resources['de.form.lastentered.column.txt'],
                                field: 'lastEntered',
                                width: '100px',
                                formatter: me.formatDateColumn
                            },
                            {
                                name: resources['de.form.status.column.txt'],
                                field: 'status',
                                width: '100px',
                                formatter: me.formatStatusColumn
                            },
                            {
                                name: resources['de.form.approver.column.txt'],
                                field: 'selectedApproverUserIds',
                                width: '100px',
                                formatter: me.formatApproverColumn
                            },
                            {
                                name: resources['de.form.owner.column.txt'],
                                field: 'owner',
                                width: '100px',
                                formatter: me.formatOwnerColumn
                            }
                        ],
                        autoWidth: true,
                        autoHeight: true
                    },
                    'formGrid');

                grid.startup();

                // Wire up listeners

                grid.on('RowDblClick', function(event) {
                    topic.publish(AppConst.FORM_DOUBLE_CLICK, event);
                }, true);

                grid.on('selected', function(event) {
                    topic.publish(AppConst.FORM_CLICK, event);
                }, true);

                // Disable appropriate buttons if user doesn't have write access, don't both wiring them up
                if (!GeneralAppConst.CAN_UPDATE) {
                    getNewButton().set('disabled', true);
                    getCopyButton().set('disabled', true);
                    getDeleteButton().set('disabled', true);
                    getStartWorkflowButton().set('disabled', true);
                    getStopRepeatButton().set('disabled', true);
                    getAbortWorkflowButton().set('disabled', true);
                }
                else {
                    getNewButton().on('click', function() {
                        topic.publish(AppConst.NEW_FORM_CLICK);
                    });

                    getCopyButton().on('click', function() {
                        topic.publish(AppConst.COPY_FORM_CLICK);
                    });

                    getDeleteButton().on('click', function() {
                        topic.publish(AppConst.DELETE_FORM_CLICK);
                    });

                    getStartWorkflowButton().on('click', function() {
                        topic.publish(AppConst.START_WORKFLOW_CLICK);
                    });

                    getStopRepeatButton().on('click', function() {
                        topic.publish(AppConst.STOP_WORKFLOW_CLICK);
                    });

                    getAbortWorkflowButton().on('click', function() {
                        topic.publish(AppConst.ABORT_WORKFLOW_CLICK);
                    });
                }

                getEditButton().on('click', function() {
                    topic.publish(AppConst.EDIT_FORM_CLICK);
                });

                getRefreshFormsButton().on('click', function() {
                    topic.publish(AppConst.REFRESH_FORMS_CLICK);
                });
            }, function() {
                new DialogTemplate({
                    type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                }).show();
            });
        },
        formatDateColumn: function(date) {
            return locale.format(new Date(date),{
                formatLength: 'short'
            });
        },
        formatStatusColumn: function(status) {
            switch(status) {
                case AppConst.DATA_ENTRY_STATUS_NO_WORKFLOW:
                    return resources['de.form.status.no.workflow.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_NOT_STARTED:
                    return resources['de.form.status.workflow.not.started.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_IN_PROGRESS:
                    return resources['de.form.status.ready.entry.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_COMPLETE:
                    return resources['de.form.status.partial.entry.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_AWAITING_APPROVAL:
                    return resources['de.form.status.awaiting.approval.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_PARTIAL_APPROVED:
                    return resources['de.form.status.partial.approval.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_APPROVED:
                    return resources['de.form.status.approved.txt'];
                    break;
                case AppConst.DATA_ENTRY_STATUS_WORKFLOW_DRAFT:
                    return resources['de.form.status.draft.txt'];
                    break;
            }
        },
        formatApproverColumn: function(approvers) {
            return array.map(approvers, function(approver) {
                return approver.name;
            });
        },
        formatOwnerColumn: function(owner) {
            return owner.name;
        },
        //
        // Accessors
        //
        getStartWorkflowButton: function() {
            return registry.byId('toolbarStartWorkflow');
        },
        getStopRepeatButton: function() {
            return registry.byId('toolbarStopRepeat');
        },
        getAbortWorkflowButton: function() {
            return registry.byId('toolbarAbortWorkflow');
        }
    });

});