define([
    'dojo/i18n!html/nls/STMResources',
'dojo/text!html/app/view/managerules/ManageRulesAppView.html',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/when',
    'dojo/date',
    'dojo/string',
    'dojo/date/locale',
    'dojo/topic',
    'dojo/data/ObjectStore',
    'dojo/store/Memory',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetsInTemplateMixin',
    'dojox/widget/DialogSimple',
    'dijit/registry',
    'dijit/Toolbar',
    'dijit/ToolbarSeparator',
    'dijit/layout/BorderContainer',
    'dijit/layout/ContentPane',
    'dijit/form/Button',
    'dojox/grid/DataGrid',
    'STM/view/DialogTemplate',
    'STM/model/managerules/ManageRulesRequests',
    'STM/util/ApplicationConstants',
    'STM/util/managerules/ApplicationConstants'
], function(resources, templateString, declare, array, when, date, string, locale, topic, ObjectStore, Memory,
            _TemplatedWidget, _WidgetsInTemplateMixin, DialogSimple, registry, Toolbar, ToolbarSeparator, BorderContainer,
            ContentPane, Button, DataGrid, DialogTemplate, ManageRulesRequests, GeneralAppConst, AppConst) {

    //
    // Private getters for convenience
    //

    var getNewButton = function() {
        return registry.byId('toolbarNew');
    };

    var getEditButton = function() {
        return registry.byId('toolbarEditRule');
    };

    var getCopyButton = function() {
        return registry.byId('toolbarCopy');
    };

    var getDeleteButton = function() {
        return registry.byId('toolbarDelete');
    };
    var substr = "<a href='" + GeneralAppConst.WEBDOC_HTML5_USER_GUIDE_RULE_URL + "' target=\"_blank\">" + resources['rule.popup.sub.txt'] + "</a>";

    templateString = string.substitute(templateString, {
        toolbarNewTooltipLabel: resources['rule.toolbar.new.txt'],
        toolbarEditTooltipLabel: resources['rule.toolbar.edit.txt'],
        toolbarSaveTooltipLabel: resources['rule.toolbar.save.txt'],
        toolbarCopyTooltipLabel: resources['rule.toolbar.copy.txt'],
        toolbarDeleteTooltipLabel: resources['rule.toolbar.delete.txt'],
        helplabel: resources['help.txt'],
    	helpRuleTeaser: resources['rule.teaser.txt'],
        helpRuleContext: string.substitute(resources['rule.popup.txt'], [substr] ),
        helpText: resources['rule.helptip.txt']
    });

    return declare([BorderContainer, _TemplatedWidget, _WidgetsInTemplateMixin], {
     //   'class': 'stmFixedHeightContent',
        templateString: templateString,
        //
        // Overrides and init stuff
        //
        constructor: function(args) {
            args['id'] = 'manageRulesAppView';
        },
        postCreate: function() {
            var me = this;

            me.initUI();
        },
        initUI: function() {
            var me = this;

            when(ManageRulesRequests.getRuleList(), function(rules) {
	            var store = new ObjectStore({
                    objectStore: new Memory({data: rules})
	            });

	            var grid = new DataGrid({
	                store: store,
	                structure: [
	                    {name: resources['rule.name.column.txt'], field: 'name', width: '200px'},
	                    {name: resources['rule.desc.column.txt'], field: 'description', width: '400px'}
	                ],
	                autoWidth: true,
	                autoHeight: true
	                },
	                'ruleGrid');

	            grid.startup();

	            // Wire up listeners

	            grid.on('RowDblClick', function(event) {
	                topic.publish(AppConst.RULE_DOUBLE_CLICK, event);
	            }, true);

                grid.on('selected', function(event) {
                    topic.publish(AppConst.RULE_CLICK, event);
                }, true);

                // Disable appropriate buttons if user doesn't have write access, don't both wiring them up
                if (!GeneralAppConst.CAN_UPDATE) {
                    getNewButton().set('disabled', true);
                    getCopyButton().set('disabled', true);
                    getDeleteButton().set('disabled', true);
                }
                else {
                    getNewButton().on('click', function() {
                        topic.publish(AppConst.NEW_RULE_CLICK);
                    });

                    getCopyButton().on('click', function() {
                        topic.publish(AppConst.COPY_RULE_CLICK);
                    });

                    getDeleteButton().on('click', function() {
                        topic.publish(AppConst.DELETE_RULE_CLICK);
                    });
                }

                getEditButton().on('click', function() {
                    topic.publish(AppConst.EDIT_RULE_CLICK);
                });


            }, function() {
                new DialogTemplate({
                    type: GeneralAppConst.ERROR_SERVER_DATA_DIALOG
                }).show();
            });
         }
    });

});
